"""This module is the entry point for ostorlab CLI."""
from typing import Optional

import click


@click.group()
@click.pass_context
@click.option('--proxy', '-X', help='Proxy to route HTTPS requests through.')
@click.option('--tlsverify/--no-tlsverify', help='Control TLS server certificate verification.', default=True)
def rootcli(ctx: click.core.Context, proxy: Optional[str] = None, tlsverify: Optional[bool] = True) -> None:
    """Ostorlab is an open-source project to help automate security testing.\n
    Ostorlab standardizes interoperability between tools in a consistent, scalable, and performant way."""
    ctx.obj = {}
    ctx.obj['proxy'] = proxy
    ctx.obj['tlsverify'] = tlsverify



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: v3/asset/file/ios/ipa/ipa.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='v3/asset/file/ios/ipa/ipa.proto',
  package='v3.asset.file.ios.ipa',
  syntax='proto2',
  serialized_pb=_b('\n\x1fv3/asset/file/ios/ipa/ipa.proto\x12\x15v3.asset.file.ios.ipa\"(\n\x07Message\x12\x0f\n\x07\x63ontent\x18\x01 \x02(\x0c\x12\x0c\n\x04path\x18\x02 \x01(\t')
)
_sym_db.RegisterFileDescriptor(DESCRIPTOR)




_MESSAGE = _descriptor.Descriptor(
  name='Message',
  full_name='v3.asset.file.ios.ipa.Message',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='content', full_name='v3.asset.file.ios.ipa.Message.content', index=0,
      number=1, type=12, cpp_type=9, label=2,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='path', full_name='v3.asset.file.ios.ipa.Message.path', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=58,
  serialized_end=98,
)

DESCRIPTOR.message_types_by_name['Message'] = _MESSAGE

Message = _reflection.GeneratedProtocolMessageType('Message', (_message.Message,), dict(
  DESCRIPTOR = _MESSAGE,
  __module__ = 'v3.asset.file.ios.ipa.ipa_pb2'
  # @@protoc_insertion_point(class_scope:v3.asset.file.ios.ipa.Message)
  ))
_sym_db.RegisterMessage(Message)


# @@protoc_insertion_point(module_scope)



"""Scan module that handles running a scan using a list of agent keys and a target asset."""

import click

from ostorlab.cli.rootcli import rootcli
from ostorlab.runtimes import registry


@rootcli.group()
@click.option('--runtime', type=click.Choice(['local', 'remote', 'hybrid']),
              help="""Runtime is in charge of preparing the environment to trigger a scan.\n
                    Specify which runtime to use: \n
                    local: on you local machine\n
                    remote: on Ostorlab cloud, (requires login)\n
                    hybrid: running partially on Ostorlab cloud and partially on the local machine\n
                   """,
              required=True)
@click.pass_context
def scan(ctx: click.core.Context, runtime: str) -> None:
    """You can use scan [subcommand] to list, start or stop a scan.\n
    Examples:\n
        - Show list of scans: ostorlab scan --list\n
        - Show full details of a scan: ostorlab scan describe --scan=scan_id.\n
    """
    try:
        runtime_instance = registry.select_runtime(runtime)
        ctx.obj['runtime'] = runtime_instance
    except registry.RuntimeNotFoundError as e:
        raise click.ClickException(f'The selected runtime {runtime} is not supported.') from e



"""Module for the command run inside the group scan.
This module takes care of preparing the selected runtime and the lists of provided agents, before starting a scan.
Example of usage:
    - ostorlab scan run --agents=agent1,agent2 --title=test_scan [asset] [options]."""
import io
from typing import List
import click
from ostorlab.runtimes import definitions
from ostorlab.cli.scan import scan


@scan.group()
@click.option('--agents', multiple=True, help='List of agents keys. to use in the scan. ', required=True)
@click.option('--title', '-t', help='Scan title.')
@click.option('--agent-group-definition', '-g', type=click.File('r'),
              help='Path to agents group definition file (yaml).')
@click.pass_context
def run(ctx: click.core.Context, agents: List[str], agent_group_definition: io.FileIO,
        title: str) -> None:
    """Start a new scan on a specific asset.\n
    Usage:\n
        - ostorlab scan run --agents=agent1,agent2 --title=test_scan android-apk --file=path/to/application.apk
    """
    if agents:
        agents_settings: List[definitions.AgentSettings] = []
        for agent_key in agents:
            agents_settings.append(
                definitions.AgentSettings(key=agent_key))
        agent_group = definitions.AgentGroupDefinition(agents=agents_settings)
    elif agent_group_definition:
        agent_group = definitions.AgentGroupDefinition.from_yaml(agent_group_definition)
    else:
        raise click.ClickException('Missing agent list or agent group definition.')

    runtime_instance = ctx.obj['runtime']
    if runtime_instance.can_run(agent_group_definition=agent_group):
        ctx.obj['agent_group_definition'] = agent_group
        ctx.obj['title'] = title
    else:
        raise click.ClickException('Runtime do not support provided agent list or group definition.')
